@@ -1,15 +1,21 @@
 /*
-   Copyright (c) 2003-2008 MySQL AB
-   Use is subject to license terms.
+   Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -21,6 +27,9 @@
 
 #include "SignalData.hpp"
 
+#define JAM_FILE_ID 162
+
+
 class KeyInfo {
   /**
    * Sender(s)
@@ -72,12 +81,15 @@ private:
     1 word of AttributeHeader (containing attribute Id and byte length)
     N words of attribute data (N = (length+3)>>2).
   Additionally, it is possible to send multiple range bounds in a single
-  SCAN_TABREQ and associated ATTRINFO stream (using NdbRecord Index scans and
+  SCAN_TABREQ and associated KEYINFO stream (using NdbRecord Index scans and
   multiple calls to setBound with different range numbers). In this case, the 
   first word of each range bound contains additional information: 
-  bits 16-31 holds the length of this bound, in words of ATTRINFO data, 
+  bits 16-31 holds the length of this bound, in words of KEYINFO data,
   and bits 4-15 holds a number RANGE_NO specified by the application that 
   can be read back from the RANGE_NO pseudo-column.
 
 */
+
+#undef JAM_FILE_ID
+
 #endif
