@@ -1,18 +1,26 @@
-/* Copyright (c) 2003, 2005 MySQL AB
-
+/*
+   Copyright (c) 2003, 2014, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
+*/
 
 
 //**************************************************************************** 
@@ -45,8 +53,10 @@ public:
    *
    * The method may either execute the signal immediately (NDB API), or
    * queue it for later execution (kernel).
+   *
+   * @returns true if no more signals should be delivered
    */
-  virtual void deliver_signal(SignalHeader * const header,
+  virtual bool deliver_signal(SignalHeader * const header,
                               Uint8 prio,
                               Uint32 * const signalData,
                               LinearSectionPtr ptr[3]) = 0;
@@ -111,6 +121,16 @@ public:
    *
    */
   virtual ~TransporterReceiveHandle() { };
+
+#ifndef NDEBUG
+  /**
+   * 'm_active' is used by 'class TransporterReceiveWatchdog' in 
+   * DEBUG to detect concurrent calls to ::update_connections and
+   * ::performReceive() which isn't allowed.
+   */
+  TransporterReceiveHandle() : m_active(false) {};
+  volatile bool m_active;
+#endif
 };
 
 /**
