@@ -1,21 +1,30 @@
-/* Copyright (C) 2007 MySQL AB
+/*
+   Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+*/
 
 
 /*
-  ndbapi_blob.cpp:
+  ndbapi_blob_ndbrecord
 
   Illustrates the manipulation of BLOB (actually TEXT in this example).
   This example uses the NdbRecord style way of accessing tuples.
@@ -24,7 +33,9 @@
   read/write methods.
  */
 
-
+#ifdef _WIN32
+#include <winsock2.h>
+#endif
 #include <mysql.h>
 #include <mysqld_error.h>
 #include <NdbApi.hpp>
@@ -32,7 +43,9 @@
 #include <iostream>
 #include <stdio.h>
 #include <ctype.h>
-
+#include <stdlib.h>
+#include <stddef.h>
+#include <string.h>
 
 /**
  * Helper debugging macros
@@ -112,15 +125,25 @@ const NdbRecord *key_record;                    // For specifying table key
 const NdbRecord *blob_record;                   // For accessing blob
 const NdbRecord *full_record;                   // All columns, for insert
 
-/* The row is 4 bytes of primary key + space for blob handle pointer. */
-#define ROWSIZE (4 + sizeof(NdbBlob *))
+/* C struct representing the row layout */
+struct MyRow
+{
+  unsigned int myId;
+
+  /* Pointer to Blob handle for operations on the blob column
+   * Space must be left for it in the row, but a pointer to the 
+   * blob handle can also be obtained via calls to 
+   * NdbOperation::getBlobHandle()
+   */
+  NdbBlob* myText;
+};
 
 static void setup_records(Ndb *myNdb)
 {
   NdbDictionary::RecordSpecification spec[2];
 
   NdbDictionary::Dictionary *myDict= myNdb->getDictionary();
-  const NdbDictionary::Table *myTable= myDict->getTable("my_text");
+  const NdbDictionary::Table *myTable= myDict->getTable("api_blob_ndbrecord");
   if (myTable == NULL)
     APIERROR(myDict->getNdbError());
   const NdbDictionary::Column *col1= myTable->getColumn("my_id");
@@ -131,11 +154,11 @@ static void setup_records(Ndb *myNdb)
     APIERROR(myDict->getNdbError());
 
   spec[0].column= col1;
-  spec[0].offset= 0;
+  spec[0].offset= offsetof(MyRow, myId);
   spec[0].nullbit_byte_offset= 0;
   spec[0].nullbit_bit_in_byte= 0;
   spec[1].column= col2;
-  spec[1].offset= 4;
+  spec[1].offset= offsetof(MyRow, myText);
   spec[1].nullbit_byte_offset= 0;
   spec[1].nullbit_bit_in_byte= 0;
 
@@ -155,7 +178,7 @@ static void setup_records(Ndb *myNdb)
 */
 void drop_table(MYSQL &mysql)
 {
-  if (mysql_query(&mysql, "DROP TABLE my_text"))
+  if (mysql_query(&mysql, "DROP TABLE api_blob_ndbrecord"))
     MYSQLERROR(mysql);
 }
 
@@ -167,7 +190,7 @@ int try_create_table(MYSQL &mysql)
 {
   return mysql_query(&mysql,
                      "CREATE TABLE"
-                     "  my_text"
+                     "  api_blob_ndbrecord"
                      "    (my_id INT UNSIGNED NOT NULL,"
                      "     my_text TEXT NOT NULL,"
                      "     PRIMARY KEY USING HASH (my_id))"
@@ -180,7 +203,7 @@ void create_table(MYSQL &mysql)
   {
     if (mysql_errno(&mysql) != ER_TABLE_EXISTS_ERROR)
       MYSQLERROR(mysql);
-    std::cout << "MySQL Cluster already has example table: my_text. "
+    std::cout << "MySQL Cluster already has example table: api_blob_ndbrecord. "
               << "Dropping it..." << std::endl;
     /******************
      * Recreate table *
@@ -193,17 +216,17 @@ void create_table(MYSQL &mysql)
 
 int populate(Ndb *myNdb)
 {
-  char row[ROWSIZE];
+  MyRow row;
 
   NdbTransaction *myTrans= myNdb->startTransaction();
   if (myTrans == NULL)
     APIERROR(myNdb->getNdbError());
 
-  Uint32 id= 1;
-  memcpy(&row[0], &id, 4);
-  NdbOperation *myNdbOperation= myTrans->insertTuple(full_record, row);
+  row.myId= 1;
+  const NdbOperation *myNdbOperation= myTrans->insertTuple(full_record, (const char*) &row);
   if (myNdbOperation == NULL)
     APIERROR(myTrans->getNdbError());
+
   NdbBlob *myBlobHandle= myNdbOperation->getBlobHandle("my_text");
   if (myBlobHandle == NULL)
     APIERROR(myNdbOperation->getNdbError());
@@ -217,7 +240,7 @@ int populate(Ndb *myNdb)
 
 int update_key(Ndb *myNdb)
 {
-  char row[ROWSIZE];
+  MyRow row;
 
   /*
     Uppercase all characters in TEXT field, using primary key operation.
@@ -229,17 +252,23 @@ int update_key(Ndb *myNdb)
   if (myTrans == NULL)
     APIERROR(myNdb->getNdbError());
 
-  Uint32 id= 1;
-  memcpy(&row[0], &id, 4);
-  NdbOperation *myNdbOperation=
-    myTrans->updateTuple(key_record, row, blob_record, row);
+  row.myId= 1;
+
+  const NdbOperation *myNdbOperation=
+    myTrans->updateTuple(key_record, 
+                         (const char*) &row, 
+                         blob_record, 
+                         (const char*) &row);
   if (myNdbOperation == NULL)
     APIERROR(myTrans->getNdbError());
+
   NdbBlob *myBlobHandle= myNdbOperation->getBlobHandle("my_text");
   if (myBlobHandle == NULL)
     APIERROR(myNdbOperation->getNdbError());
 
-  /* Execute NoCommit to make the blob handle active. */
+  /* Execute NoCommit to make the blob handle active so 
+   * that we can determine the actual Blob length
+   */
   if (-1 == myTrans->execute(NdbTransaction::NoCommit))
     APIERROR(myTrans->getNdbError());
 
@@ -299,7 +328,6 @@ int update_scan(Ndb *myNdb)
     updateCurrentTuple().
   */
   char buffer[10000];
-  char row[ROWSIZE];
 
   NdbTransaction *myTrans= myNdb->startTransaction();
   if (myTrans == NULL)
@@ -319,11 +347,11 @@ int update_scan(Ndb *myNdb)
   if (-1 == myTrans->execute(NdbTransaction::NoCommit))
     APIERROR(myTrans->getNdbError());
 
-  const char *out_row;
+  const MyRow *out_row;
   int res;
   for (;;)
   {
-    res= myScanOp->nextResult(out_row, true);
+    res= myScanOp->nextResult((const char**)&out_row, true, false);
     if (res==1)
       break;                                    // Scan done.
     else if (res)
@@ -337,8 +365,13 @@ int update_scan(Ndb *myNdb)
     for (Uint64 j= 0; j < length; j++)
       buffer[j]= tolower(buffer[j]);
 
-    NdbOperation *myUpdateOp=
-      myScanOp->updateCurrentTuple(myTrans, blob_record, row);
+    /* 'Take over' the row locks from the scan to a separate
+     * operation for updating the tuple
+     */
+    const NdbOperation *myUpdateOp=
+      myScanOp->updateCurrentTuple(myTrans, 
+                                   blob_record, 
+                                   (const char*)out_row);
     if (myUpdateOp == NULL)
       APIERROR(myTrans->getNdbError());
     NdbBlob *myBlobHandle2= myUpdateOp->getBlobHandle("my_text");
@@ -375,8 +408,18 @@ int myFetchHook(NdbBlob* myBlobHandle, void* arg)
 
 int fetch_key(Ndb *myNdb)
 {
-  char key_row[ROWSIZE];
-  char out_row[ROWSIZE];
+  /* Fetch a blob without specifying how many bytes
+   * to read up front, in one execution using
+   * the 'ActiveHook' mechanism.
+   * The supplied ActiveHook procedure is called when
+   * the Blob handle becomes 'active'.  At that point
+   * the length of the Blob can be obtained, and buffering
+   * arranged, and the data read requested.
+   */
+
+  /* Separate rows used to specify key and hold result */
+  MyRow key_row;
+  MyRow out_row;
 
   /*
     Fetch and show the blob field, using setActiveHook().
@@ -386,13 +429,21 @@ int fetch_key(Ndb *myNdb)
   if (myTrans == NULL)
     APIERROR(myNdb->getNdbError());
 
-  Uint32 id= 1;
-  memcpy(&key_row[0], &id, 4);
-  NdbOperation *myNdbOperation=
-    myTrans->readTuple(key_record, key_row, blob_record, out_row);
+  key_row.myId= 1;
+  out_row.myText= NULL;
+  const NdbOperation *myNdbOperation=
+    myTrans->readTuple(key_record, 
+                       (const char*) &key_row, 
+                       blob_record, 
+                       (char*) &out_row);
   if (myNdbOperation == NULL)
     APIERROR(myTrans->getNdbError());
-  NdbBlob *myBlobHandle= myNdbOperation->getBlobHandle("my_text");
+
+  /* This time, we'll get the blob handle from the row, because
+   * we can.  Alternatively, we could use the normal mechanism
+   * of calling getBlobHandle().
+   */
+  NdbBlob *myBlobHandle= out_row.myText;
   if (myBlobHandle == NULL)
     APIERROR(myNdbOperation->getNdbError());
   struct ActiveHookData ahd;
@@ -419,18 +470,22 @@ int fetch_key(Ndb *myNdb)
 int update2_key(Ndb *myNdb)
 {
   char buffer[10000];
-  char row[ROWSIZE];
+  MyRow row;
 
-  /* Simple setValue() update. */
+  /* Simple setValue() update specified before the
+   * Blob handle is made active
+   */
 
   NdbTransaction *myTrans= myNdb->startTransaction();
   if (myTrans == NULL)
     APIERROR(myNdb->getNdbError());
 
-  Uint32 id= 1;
-  memcpy(&row[0], &id, 4);
-  NdbOperation *myNdbOperation=
-    myTrans->updateTuple(key_record, row, blob_record, row);
+  row.myId= 1;
+  const NdbOperation *myNdbOperation=
+    myTrans->updateTuple(key_record, 
+                         (const char*)&row, 
+                         blob_record, 
+                         (char*) &row);
   if (myNdbOperation == NULL)
     APIERROR(myTrans->getNdbError());
   NdbBlob *myBlobHandle= myNdbOperation->getBlobHandle("my_text");
@@ -450,17 +505,18 @@ int update2_key(Ndb *myNdb)
 
 int delete_key(Ndb *myNdb)
 {
-  char row[ROWSIZE];
+  MyRow row;
 
-  /* Deletion of blob row. */
+  /* Deletion of row containing blob via primary key. */
 
   NdbTransaction *myTrans= myNdb->startTransaction();
   if (myTrans == NULL)
     APIERROR(myNdb->getNdbError());
 
-  Uint32 id= 1;
-  memcpy(&row[0], &id, 4);
-  NdbOperation *myNdbOperation= myTrans->deleteTuple(key_record, row);
+  row.myId= 1;
+  const NdbOperation *myNdbOperation= myTrans->deleteTuple(key_record, 
+                                                           (const char*)&row, 
+                                                           full_record);
   if (myNdbOperation == NULL)
     APIERROR(myTrans->getNdbError());
 
@@ -494,8 +550,8 @@ int main(int argc, char**argv)
                              0, mysqld_sock, 0) )
       MYSQLERROR(mysql);
 
-    mysql_query(&mysql, "CREATE DATABASE TEST_DB");
-    if (mysql_query(&mysql, "USE TEST_DB") != 0)
+    mysql_query(&mysql, "CREATE DATABASE ndb_examples");
+    if (mysql_query(&mysql, "USE ndb_examples") != 0)
       MYSQLERROR(mysql);
 
     create_table(mysql);
@@ -516,7 +572,7 @@ int main(int argc, char**argv)
     exit(-1);
   }
 
-  Ndb myNdb(&cluster_connection,"TEST_DB");
+  Ndb myNdb(&cluster_connection,"ndb_examples");
   if (myNdb.init(1024) == -1) {      // Set max 1024 parallel transactions
     APIERROR(myNdb.getNdbError());
     exit(-1);
@@ -542,8 +598,5 @@ int main(int argc, char**argv)
   if(delete_key(&myNdb) > 0)
     std::cout << "delete_key: Success!" << std::endl;
 
-  /* Drop table. */
-  drop_table(mysql);
-
   return 0;
 }
