@@ -1,19 +1,26 @@
-/* Copyright (c) 2003-2007 MySQL AB
-
+/*
+   Copyright (c) 2003, 2015, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
-
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+*/
 
 #include <GlobalSignalNumbers.h>
 
@@ -259,8 +266,7 @@ const GsnName SignalNames [] = {
   ,{ GSN_NEXT_SCANREF,           "NEXT_SCANREF" }
   ,{ GSN_NEXT_SCANREQ,           "NEXT_SCANREQ" }
   ,{ GSN_NF_COMPLETEREP,         "NF_COMPLETEREP" }
-//  ,{ GSN_OPEN_COMCONF,           "OPEN_COMCONF" }
-//  ,{ GSN_OPEN_COMREF,            "OPEN_COMREF" }
+  ,{ GSN_EXPAND_CLNT,            "EXPAND_CLNT" }
   ,{ GSN_OPEN_COMORD,            "OPEN_COMORD" }
   ,{ GSN_PACKED_SIGNAL,          "PACKED_SIGNAL" }
   ,{ GSN_PREP_FAILCONF,          "PREP_FAILCONF" }
@@ -438,6 +444,7 @@ const GsnName SignalNames [] = {
   ,{ GSN_DROP_TRIG_CONF,         "DROP_TRIG_CONF" }
   ,{ GSN_DROP_TRIG_REF,          "DROP_TRIG_REF" }
   ,{ GSN_FIRE_TRIG_ORD,          "FIRE_TRIG_ORD" }
+  ,{ GSN_FIRE_TRIG_ORD_L,        "FIRE_TRIG_ORD_L" }
   ,{ GSN_TRIG_ATTRINFO,          "TRIG_ATTRINFO" }
 
   ,{ GSN_CREATE_INDX_REQ,        "CREATE_INDX_REQ" }
@@ -696,6 +703,8 @@ const GsnName SignalNames [] = {
   ,{ GSN_BUILD_INDX_IMPL_CONF, "BUILD_INDX_IMPL_CONF" }
   ,{ GSN_BUILD_INDX_IMPL_REF, "BUILD_INDX_IMPL_REF" }
 
+  ,{ GSN_NODE_STARTED_REP, "NODE_STARTED_REP" }
+
   ,{ GSN_RESTORE_LCP_REQ, "RESTORE_LCP_REQ" }
   ,{ GSN_RESTORE_LCP_CONF, "RESTORE_LCP_CONF" }
   ,{ GSN_RESTORE_LCP_REF, "RESTORE_LCP_REF" }
@@ -776,8 +785,32 @@ const GsnName SignalNames [] = {
   ,{ GSN_GET_CONFIG_REF, "GET_CONFIG_REF" }
   ,{ GSN_GET_CONFIG_CONF, "GET_CONFIG_CONF" }
 
+  ,{ GSN_ALLOC_NODEID_REQ, "ALLOC_NODEID_REQ" }
+  ,{ GSN_ALLOC_NODEID_CONF, "ALLOC_NODEID_CONF" }
+  ,{ GSN_ALLOC_NODEID_REF, "ALLOC_NODEID_REF" }
+
   ,{ GSN_LCP_STATUS_REQ, "LCP_STATUS_REQ" }
   ,{ GSN_LCP_STATUS_CONF, "LCP_STATUS_CONF" }
   ,{ GSN_LCP_STATUS_REF, "LCP_STATUS_REF" }
+
+  ,{ GSN_CREATE_FK_REQ, "CREATE_FK_REQ" }
+  ,{ GSN_CREATE_FK_REF, "CREATE_FK_REF" }
+  ,{ GSN_CREATE_FK_CONF, "CREATE_FK_CONF" }
+  ,{ GSN_DROP_FK_REQ, "DROP_FK_REQ" }
+  ,{ GSN_DROP_FK_REF, "DROP_FK_REF" }
+  ,{ GSN_DROP_FK_CONF, "DROP_FK_CONF" }
+  ,{ GSN_CREATE_FK_IMPL_REQ, "CREATE_FK_IMPL_REQ" }
+  ,{ GSN_CREATE_FK_IMPL_REF, "CREATE_FK_IMPL_REF" }
+  ,{ GSN_CREATE_FK_IMPL_CONF, "CREATE_FK_IMPL_CONF" }
+  ,{ GSN_DROP_FK_IMPL_REQ, "DROP_FK_IMPL_REQ" }
+  ,{ GSN_DROP_FK_IMPL_REF, "DROP_FK_IMPL_REF" }
+  ,{ GSN_DROP_FK_IMPL_CONF, "DROP_FK_IMPL_CONF" }
+  ,{ GSN_BUILD_FK_REQ, "BUILD_FK_REQ" }
+  ,{ GSN_BUILD_FK_REF, "BUILD_FK_REF" }
+  ,{ GSN_BUILD_FK_CONF, "BUILD_FK_CONF" }
+  ,{ GSN_BUILD_FK_IMPL_REQ, "BUILD_FK_IMPL_REQ" }
+  ,{ GSN_BUILD_FK_IMPL_REF, "BUILD_FK_IMPL_REF" }
+  ,{ GSN_BUILD_FK_IMPL_CONF, "BUILD_FK_IMPL_CONF" }
+  ,{ GSN_ISOLATE_ORD, "ISOLATE_ORD" }
 };
 const unsigned short NO_OF_SIGNAL_NAMES = sizeof(SignalNames)/sizeof(GsnName);
