@@ -1,19 +1,26 @@
-/* Copyright (c) 2003, 2014, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2003, 2015, Oracle and/or its affiliates. All rights reserved.
 
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
-
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+*/
 
 #include <ndb_global.h>
 
@@ -71,14 +78,15 @@ NdbOut& NdbOut::endline()
 {
   isHex = 0; // Reset hex to normal, if user forgot this
   m_out->println("%s", "");
-  if (m_autoflush)
-    m_out->flush();
-  return *this;
+  return flushline(false);
 }
 
-NdbOut& NdbOut::flushline()
+NdbOut& NdbOut::flushline(bool force)
 {
-  m_out->flush();
+  if (force || m_autoflush)
+  {
+    m_out->flush();
+  }
   return *this;
 }
 
@@ -90,41 +98,8 @@ NdbOut& NdbOut::setHexFormat(int _format)
 
 NdbOut& NdbOut::hexdump(const Uint32* words, size_t count)
 {
-  /**
-   * Write at most about 1000 characters.
-   * If not all words are printed end with "...\n".
-   * Words are written as "H'11223344 ", 11 character each.
-   */
   char buf[90 * 11 + 4 + 1];
-  size_t offset = 0;
-  size_t words_to_dump = count;
-  if (words_to_dump > 90)
-  {
-    words_to_dump = 90;
-  }
-  for(size_t i = 0 ; i < words_to_dump ; i ++ )
-  {
-    // Write at most 6 words per line
-    char sep = (i % 6 == 5) ? '\n' : ' ';
-    assert(offset + 11 < sizeof(buf));
-    int n = BaseString::snprintf(buf + offset, sizeof(buf) - offset, "H'%08x%c", words[i], sep);
-    assert(n == 11);
-    offset += n;
-  }
-  if (words_to_dump < count)
-  {
-    assert(offset + 4 < sizeof(buf));
-    int n = BaseString::snprintf(buf + offset, sizeof(buf) - offset, "...\n");
-    assert(n == 4);
-    offset += n;
-  }
-  else
-  {
-    assert(offset + 1 < sizeof(buf));
-    int n = BaseString::snprintf(buf + offset, sizeof(buf) - offset, "\n");
-    assert(n == 1);
-    offset += n;
-  }
+  size_t offset = BaseString::hexdump(buf, sizeof(buf), words, count);
   m_out->write(buf, offset);  
   return *this;
 }
@@ -151,39 +126,83 @@ NdbOut::~NdbOut()
 }
 
 void
-NdbOut::print(const char * fmt, ...){
+NdbOut::print(const char * fmt, ...)
+{
+  if (fmt == NULL)
+  {
+    /*
+     Function was called with fmt being NULL, this is an error
+     but handle it gracefully by simpling printing nothing
+     instead of continuing down the line whith the NULL pointer.
+
+     Catch problem with an assert in debug compile.
+    */
+    assert(false);
+    return;
+  }
+
   va_list ap;
   char buf[1000];
   
   va_start(ap, fmt);
-  if (fmt != 0)
-    BaseString::vsnprintf(buf, sizeof(buf)-1, fmt, ap);
+  BaseString::vsnprintf(buf, sizeof(buf)-1, fmt, ap);
   *this << buf;
   va_end(ap);
 }
 
 void
-NdbOut::println(const char * fmt, ...){
+NdbOut::println(const char * fmt, ...)
+{
+  if (fmt == NULL)
+  {
+    /*
+     Function was called with fmt being NULL, this is an error
+     but handle it gracefully by simpling printing nothing
+     instead of continuing down the line whith the NULL pointer.
+
+     Catch problem with an assert in debug compile.
+    */
+    assert(false);
+    *this << endl;
+    return;
+  }
+
   va_list ap;
   char buf[1000];
   
   va_start(ap, fmt);
-  if (fmt != 0)
-    BaseString::vsnprintf(buf, sizeof(buf)-1, fmt, ap);
-  *this << buf << endl;
+  size_t len = BaseString::vsnprintf(buf, sizeof(buf)-1, fmt, ap);
+  if (len > sizeof(buf) - 2) len = sizeof(buf) - 2;
+  memcpy(&buf[len], "\n", 2);
+  *this << buf;
+  flushline(false);
   va_end(ap);
 }
 
-extern "C"
+static
 void 
-vndbout_c(const char * fmt, va_list ap){
-  char buf[1000];
-  
-  if (fmt != 0)
+vndbout_c(const char * fmt, va_list ap)
+{
+  if (fmt == NULL)
   {
-    BaseString::vsnprintf(buf, sizeof(buf)-1, fmt, ap);
+    /*
+     Function was called with fmt being NULL, this is an error
+     but handle it gracefully by simpling printing an empty newline
+     instead of continuing down the line whith the NULL pointer.
+
+     Catch problem with an assert in debug compile.
+    */
+    assert(false);
+    ndbout << endl; // Empty newline
+    return;
   }
-  ndbout << buf << endl;
+
+  char buf[1000];
+  size_t len = BaseString::vsnprintf(buf, sizeof(buf)-1, fmt, ap);
+  if (len > sizeof(buf) - 2) len = sizeof(buf) - 2;
+  memcpy(&buf[len], "\n", 2);
+  ndbout << buf;
+  ndbout.flushline(false);
 }
 
 extern "C"
