@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2003, 2011, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2016, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -48,6 +55,10 @@
 #include <NdbSleep.h>
 #include <SafeCounter.hpp>
 #include <SectionReader.hpp>
+#include <vm/WatchDog.hpp>
+
+#define JAM_FILE_ID 380
+
 
 #define ZREPORT_MEMORY_USAGE 1000
 
@@ -161,7 +172,7 @@ Cmvmi::Cmvmi(Block_context& ctx) :
 
   c_memusage_report_frequency = 0;
 
-  m_start_time = NdbTick_CurrentMillisecond() / 1000; // seconds
+  m_start_time = NdbTick_getCurrentTicks();
 
   bzero(g_acc_pages_used, sizeof(g_acc_pages_used));
 }
@@ -603,7 +614,7 @@ Cmvmi::execEVENT_SUBSCRIBE_REQ(Signal * signal){
     /**
      * Create a new one
      */
-    if(subscribers.seize(ptr) == false){
+    if (subscribers.seizeFirst(ptr) == false){
       sendSignal(senderRef, GSN_EVENT_SUBSCRIBE_REF, signal, 1, JBB);
       return;
     }
@@ -1208,8 +1219,8 @@ void Cmvmi::execTAMPER_ORD(Signal* signal)
   }
   else if (errNo < 12000)
   {
-    // DBUTIL_REF ?
     jam();
+    tuserblockref = PGMAN_REF;
   }
   else if (errNo < 13000)
   {
@@ -1246,6 +1257,11 @@ void Cmvmi::execTAMPER_ORD(Signal* signal)
     jam();
     tuserblockref = TRIX_REF;
   }
+  else if (errNo < 20000)
+  {
+    jam();
+    tuserblockref = DBUTIL_REF;
+  }
   else if (errNo < 30000)
   {
     /*--------------------------------------------------------------------*/
@@ -1290,7 +1306,7 @@ void Cmvmi::execTAMPER_ORD(Signal* signal)
   if (tuserblockref != 0)
   {
     signal->theData[0] = errNo;
-    sendSignal(tuserblockref, GSN_NDB_TAMPER, signal, 1, JBB);
+    sendSignal(tuserblockref, GSN_NDB_TAMPER, signal, signal->getLength(), JBB);
   }
 #endif
 }//execTAMPER_ORD()
@@ -1340,6 +1356,11 @@ cmp_event_buf(const void * ptr0, const void * ptr1)
   return time0 - time1;
 }
 
+#if defined VM_TRACE || defined ERROR_INSERT
+static Uint32 f_free_segments[32];
+static Uint32 f_free_segment_pos = 0;
+#endif
+
 void
 Cmvmi::execDUMP_STATE_ORD(Signal* signal)
 {
@@ -1441,6 +1462,79 @@ Cmvmi::execDUMP_STATE_ORD(Signal* signal)
 	      g_sectionSegmentPool.getNoOfFree());
   }
 
+  if (arg == DumpStateOrd::CmvmiLongSignalMemorySnapshotStart)
+  {
+#if defined VM_TRACE || defined ERROR_INSERT
+    f_free_segment_pos = 0;
+    bzero(f_free_segments, sizeof(f_free_segments));
+#endif
+  }
+
+  if (arg == DumpStateOrd::CmvmiLongSignalMemorySnapshot)
+  {
+#if defined VM_TRACE || defined ERROR_INSERT
+    f_free_segments[f_free_segment_pos] = g_sectionSegmentPool.getNoOfFree();
+    f_free_segment_pos = (f_free_segment_pos + 1) % NDB_ARRAY_SIZE(f_free_segments);
+#endif
+  }
+
+  if (arg == DumpStateOrd::CmvmiLongSignalMemorySnapshotCheck)
+  {
+#if defined VM_TRACE || defined ERROR_INSERT
+    Uint32 start = (f_free_segment_pos + 1)% NDB_ARRAY_SIZE(f_free_segments);
+    Uint32 stop = (f_free_segment_pos - 1) % NDB_ARRAY_SIZE(f_free_segments);
+    Uint32 cnt_dec = 0;
+    Uint32 cnt_inc = 0;
+    Uint32 cnt_same = 0;
+    for (Uint32 i = start; i != stop; i = (i + 1) % NDB_ARRAY_SIZE(f_free_segments))
+    {
+      Uint32 prev = (i - 1) % NDB_ARRAY_SIZE(f_free_segments);
+      if (f_free_segments[prev] == f_free_segments[i])
+        cnt_same++;
+      else if (f_free_segments[prev] > f_free_segments[i])
+        cnt_dec++;
+      else if (f_free_segments[prev] < f_free_segments[i])
+        cnt_inc++;
+    }
+
+    printf("snapshots: ");
+    for (Uint32 i = start; i != stop; i = (i + 1) % NDB_ARRAY_SIZE(f_free_segments))
+    {
+      printf("%u ", f_free_segments[i]);
+    }
+    printf("\n");
+    printf("summary: #same: %u #increase: %u #decrease: %u\n",
+           cnt_same, cnt_inc, cnt_dec);
+
+    if (cnt_dec > 1)
+    {
+      /**
+       * If memory decreased more than once...
+       *   it must also increase atleast once
+       */
+      ndbrequire(cnt_inc > 0);
+    }
+
+    if (cnt_dec == 1)
+    {
+      // it decreased once...this is ok
+      return;
+    }
+    if (cnt_same >= (cnt_inc + cnt_dec))
+    {
+      // it was frequently the same...this is ok
+      return;
+    }
+    if (cnt_same + cnt_dec >= cnt_inc)
+    {
+      // it was frequently the same or less...this is ok
+      return;
+    }
+
+    ndbrequire(false);
+#endif
+  }
+
   if (dumpState->args[0] == DumpStateOrd::DumpPageMemory)
   {
     const Uint32 len = signal->getLength();
@@ -1461,6 +1555,19 @@ Cmvmi::execDUMP_STATE_ORD(Signal* signal)
     {
       // Dump data and index memory to specific ref
       Uint32 result_ref = signal->theData[1];
+      /* Validate ref */
+      {
+        Uint32 node = refToNode(result_ref);
+        if (node == 0 || 
+            node >= MAX_NODES)
+        {
+          ndbout_c("Bad node in ref to DUMP %u : %u %u",
+                   DumpStateOrd::DumpPageMemory,
+                   node,
+                   result_ref);
+          return;
+        }
+      }
       reportDMUsage(signal, 0, result_ref);
       reportIMUsage(signal, 0, result_ref);
       return;
@@ -1637,24 +1744,23 @@ Cmvmi::execDUMP_STATE_ORD(Signal* signal)
     sendSignal(reference(), GSN_TESTSIG, signal, numArgs, JBB);
   }
 
-#ifdef ERROR_INSERT
-  if (arg == 9000 || arg == 9002)
+  if (arg == DumpStateOrd::CmvmiSetKillerWatchdog)
   {
-    // Migrated to TRPMAN
-    sendSignal(TRPMAN_REF, GSN_DUMP_STATE_ORD, signal, signal->getLength(),JBB);
-  }
-  if (arg == 9001)
-  {
-    // Migrated to TRPMAN
-    sendSignal(TRPMAN_REF, GSN_DUMP_STATE_ORD, signal, signal->getLength(),JBB);
+    bool val = true;
+    if (signal->length() >= 2)
+    {
+      val = (signal->theData[1] != 0);
+    }
+    globalEmulatorData.theWatchDog->setKillSwitch(val);
+    return;
   }
 
+#ifdef ERROR_INSERT
   if (arg == 9004 && signal->getLength() == 2)
   {
     SET_ERROR_INSERT_VALUE(9004);
 
-    // Migrated to TRPMAN
-    sendSignal(TRPMAN_REF, GSN_DUMP_STATE_ORD, signal, signal->getLength(),JBB);
+    /* Actual handling of dump code moved to TRPMAN */
   }
 #endif
 
@@ -1683,41 +1789,6 @@ Cmvmi::execDUMP_STATE_ORD(Signal* signal)
 #endif
 #endif
 
-#ifdef ERROR_INSERT
-  /* <Target NodeId> dump 9992 <NodeId list>
-   * On Target NodeId, block receiving signals from NodeId list
-   *
-   * <Target NodeId> dump 9993 <NodeId list>
-   * On Target NodeId, resume receiving signals from NodeId list
-   *
-   * <Target NodeId> dump 9991
-   * On Target NodeId, resume receiving signals from any blocked node
-   *
-   *
-   * See also code in QMGR for blocking receive from nodes based
-   * on HB roles.
-   *
-   */
-  if((arg == 9993) ||  /* Unblock recv from nodeid */
-     (arg == 9992))    /* Block recv from nodeid */
-  {
-    // Migrated to TRPMAN
-    sendSignal(TRPMAN_REF, GSN_DUMP_STATE_ORD, signal, signal->getLength(),JBB);
-  }
-
-  if (arg == 9990) /* Block recv from all ndbd matching pattern */
-  {
-    // Migrated to TRPMAN
-    sendSignal(TRPMAN_REF, GSN_DUMP_STATE_ORD, signal, signal->getLength(),JBB);
-  }
-
-  if (arg == 9991) /* Unblock recv from all blocked */
-  {
-    // Migrated to TRPMAN
-    sendSignal(TRPMAN_REF, GSN_DUMP_STATE_ORD, signal, signal->getLength(),JBB);
-  }
-#endif
-
   if (arg == 9999)
   {
     Uint32 delay = 1000;
@@ -1848,7 +1919,8 @@ void Cmvmi::execDBINFO_SCANREQ(Signal *signal)
     jam();
     const NodeState& nodeState = getNodeState();
     const Uint32 start_level = nodeState.startLevel;
-    const NDB_TICKS uptime = (NdbTick_CurrentMillisecond()/1000) - m_start_time;
+    const NDB_TICKS now = NdbTick_getCurrentTicks();
+    const Uint64 uptime = NdbTick_Elapsed(m_start_time, now).seconds();
     Uint32 generation = m_ctx.m_config.get_config_generation(); 
  
     Ndbinfo::Row row(signal, req);
@@ -1881,6 +1953,12 @@ void Cmvmi::execDBINFO_SCANREQ(Signal *signal)
         sizeof(GlobalPage),
         0,
         { CFG_DB_DATA_MEM,0,0,0 }},
+      { "Long message buffer",
+        g_sectionSegmentPool.getUsed(),
+        g_sectionSegmentPool.getSize(),
+        sizeof(SectionSegment),
+        g_sectionSegmentPool.getUsedHi(),
+        { CFG_DB_LONG_SIGNAL_BUFFER,0,0,0 }},
       { NULL, 0,0,0,0,{ 0,0,0,0 }}
     };
 
