@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2005, 2010, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2005, 2016, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -22,8 +29,17 @@
 #include <NdbOut.hpp>
 #include "Sysfile.hpp"
 
+#define JAM_FILE_ID 358
+
+
 static int g_all = 0;
 
+inline void ndb_end_and_exit(int exitcode)
+{
+  ndb_end(0);
+  exit(exitcode);
+}
+
 void 
 usage(const char * prg){
   ndbout << "Usage " << prg 
@@ -125,7 +141,7 @@ NDB_COMMAND(printSysfile,
   ndb_init();
   if(argc < 2){
     usage(argv[0]);
-    return 0;
+    ndb_end_and_exit(0);
   }
 
   for(int i = 1; i<argc; i++){
@@ -166,5 +182,5 @@ NDB_COMMAND(printSysfile,
     delete [] buf;
     continue;
   }
-  return 0;
+  ndb_end_and_exit(0);
 }
