@@ -2,13 +2,20 @@
    Copyright (c) 2003, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -296,7 +303,7 @@ DbUtil::get_systab_tableid(Signal* signal)
 
   LinearSectionPtr ptr[1];
   ptr[0].p = buf;
-  ptr[0].sz = sizeof(NAME);
+  ptr[0].sz = sizeof(buf) / sizeof(Uint32);
   sendSignal(DBDICT_REF, GSN_GET_TABINFOREQ, signal,
              GetTabInfoReq::SignalLength, JBB, ptr,1);
 }
@@ -1265,6 +1272,7 @@ DbUtil::prepareOperation(Signal* signal,
   ndbrequire(prepPagesReader.next());
   ndbrequire(prepPagesReader.getKey() == UtilPrepareReq::OperationType);
   const Uint32 operationType = prepPagesReader.getUint32();
+  prepOpPtr.p->operationType =(UtilPrepareReq::OperationTypeValue)operationType;
   
   ndbrequire(prepPagesReader.next());
   
@@ -1556,7 +1564,7 @@ DbUtil::prepareOperation(Signal* signal,
   
   TcKeyReq::setAbortOption(requestInfo, TcKeyReq::AbortOnError);
   TcKeyReq::setKeyLength(requestInfo, tableDesc.KeyLength);  
-  switch(operationType) {
+  switch((UtilPrepareReq::OperationTypeValue)operationType) {
   case(UtilPrepareReq::Read):
     prepOpPtr.p->rsLen =
       attrLength + 
@@ -1586,6 +1594,14 @@ DbUtil::prepareOperation(Signal* signal,
     prepOpPtr.p->tckey.attrLen = 0;
     TcKeyReq::setOperationType(requestInfo, ZDELETE);
     break;
+  case(UtilPrepareReq::Probe):
+    // The number of attributes should equal the size of the primary key
+    ndbrequire(tableDesc.KeyLength == attrLength);
+    prepOpPtr.p->rsLen = 0;
+    prepOpPtr.p->noOfAttr = tableDesc.NoOfKeyAttr;
+    prepOpPtr.p->tckey.attrLen = 0;
+    TcKeyReq::setOperationType(requestInfo, ZREAD);
+    break;
   case(UtilPrepareReq::Write):
     prepOpPtr.p->rsLen = 0; 
     prepOpPtr.p->noOfAttr = tableDesc.NoOfKeyAttr + noOfNonPKAttribsStored;
@@ -2141,14 +2157,17 @@ DbUtil::execUTIL_EXECUTE_REQ(Signal* signal)
       res = keyInfo->append(bufStart + header.getHeaderSize(), 
 			    header.getDataSize());
 
-    switch (TcKeyReq::getOperationType(prepOpPtr.p->tckey.requestInfo)) {
-    case ZREAD:
+    switch (prepOpPtr.p->operationType) {
+    case UtilPrepareReq::Read:
       res &= attrInfo->append(bufStart, header.getHeaderSize());
       break;
-    case ZDELETE:
+    case UtilPrepareReq::Delete:
+    case UtilPrepareReq::Probe:
       // no attrinfo for Delete
       break;
-    default:
+    case UtilPrepareReq::Insert:
+    case UtilPrepareReq::Update:
+    case UtilPrepareReq::Write:
       res &= attrInfo->append(bufStart,
 			      header.getHeaderSize() + header.getDataSize());
     }
