@@ -1,15 +1,21 @@
 /*
-   Copyright (c) 2003-2006 MySQL AB, 2009 Sun Microsystems, Inc.
-   Use is subject to license terms.
+   Copyright (c) 2003, 2014, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -29,8 +35,15 @@
 #include <Bitmask.hpp>
 #include <mgmapi.h>
 
+#include <NdbMutex.h>
+
+#define JAM_FILE_ID 292
+
+
 template <class T> class Array;
 
+//#define ARRAY_CHUNK_GUARD
+
 /**
  * Template class used for implementing an
  *   pool of object (in an array with a free list)
@@ -38,7 +51,17 @@ template <class T> class Array;
 template <class T>
 class ArrayPool {
 public:
-  ArrayPool();
+  typedef void (CallBack)(ArrayPool<T>& pool);
+
+  /* 
+    'seizeErrorHandler' is called in case of out of memory errors. Observe 
+    that the pool is not locked when seizeErrorHandler is called.
+    A function pointer rather than a virtual function is used here, because 
+    a virtual function would require explicit instantiations of ArrayPool for 
+    all T types. That would again require all T types to define the nextChunk, 
+    lastChunk and chunkSize fields. This is curently not the case.
+  */
+  explicit ArrayPool(CallBack* seizeErrorHandler=NULL);
   ~ArrayPool();
   
   /**
@@ -59,6 +82,44 @@ public:
     return size;
   }
 
+  inline Uint32 getUsed() const {
+    return size - noOfFree;
+  }
+
+  inline Uint32 getUsedHi() const {
+    return size - noOfFreeMin;
+  }
+
+  /**
+    Set the low water mark equal to the current value, so that we can track
+    the lowest value seen after this call.
+   */
+  void resetFreeMin()
+  {
+    noOfFreeMin = noOfFree;
+  }
+
+  inline void updateFreeMin(void) {
+    if (noOfFree < noOfFreeMin)
+      noOfFreeMin = noOfFree;
+  }
+
+  inline void decNoFree(void) {
+    assert(noOfFree > 0);
+    noOfFree--;
+    updateFreeMin();
+  }
+
+  inline void decNoFree(Uint32 cnt) {
+    assert(noOfFree >= cnt);
+    noOfFree -= cnt;
+    updateFreeMin();
+  }
+
+  inline Uint32 getEntrySize() const {
+    return sizeof(T);
+  }
+
   /**
    * Update p value for ptr according to i value 
    */
@@ -126,6 +187,47 @@ public:
   }
 #endif
 
+  /**
+   * Cache+LockFun is used to make thread-local caches for ndbmtd
+   *   I.e each thread has one cache-instance, and can seize/release on this
+   *       wo/ locks
+   */
+  struct Cache
+  {
+    Cache(Uint32 a0 = 512, Uint32 a1 = 256) { m_first_free = RNIL; m_free_cnt = 0; m_alloc_cnt = a0; m_max_free_cnt = a1; }
+    void init_cache(Uint32 a0, Uint32 a1)
+    {
+      m_alloc_cnt = a0;
+      m_max_free_cnt = a1;
+    }
+    Uint32 m_first_free;
+    Uint32 m_free_cnt;
+    Uint32 m_alloc_cnt;
+    Uint32 m_max_free_cnt;
+  };
+
+  struct LockFun
+  {
+    void (* lock)(void);
+    void (* unlock)(void);
+  };
+
+  bool seize(LockFun, Cache&, Ptr<T> &);
+  void release(LockFun, Cache&, Uint32 i);
+  void release(LockFun, Cache&, Ptr<T> &);
+  void releaseList(LockFun, Cache&, Uint32 n, Uint32 first, Uint32 last);
+
+  void setChunkSize(Uint32 sz);
+#ifdef ARRAY_CHUNK_GUARD
+  void checkChunks();
+#endif
+
+protected:
+  void releaseChunk(LockFun, Cache&, Uint32 n);
+
+  bool seizeChunk(Uint32 & n, Ptr<T> &);
+  void releaseChunk(Uint32 n, Uint32 first, Uint32 last);
+
 protected:
   friend class Array<T>;
 
@@ -151,7 +253,7 @@ public:
   void releaseList(Uint32 n, Uint32 first, Uint32 last);
   //private:
 
-#ifdef DEBUG
+#if 0
   Uint32 getNoOfFree2() const {
     Uint32 c2 = size;
     for(Uint32 i = 0; i<((size + 31)>> 5); i++){
@@ -180,25 +282,42 @@ public:
 #endif
 
 protected:
-  Uint32 firstFree;
-  Uint32 size;
-  Uint32 noOfFree;
   T * theArray;
-  void * alloc_ptr;
+  Uint32 size;
+  /*
+   * Protect size and theArray which are very seldomly updated from
+   * updates of often updated variables such as firstFree, noOfFree.
+   * Protect here means to have them on separate CPU cache lines to
+   * avoid false CPU cache line sharing.
+   */
+  char protect_read_var[NDB_CL_PADSZ(sizeof(Uint32) + sizeof(void*))];
+  Uint32 firstFree;
+  Uint32 noOfFree;
+  Uint32 noOfFreeMin;
+#ifdef ARRAY_GUARD
+  bool chunk;
   Uint32 bitmaskSz;
   Uint32 *theAllocatedBitmask;
+#endif
+  void * alloc_ptr;
+  // Call this function if a seize request fails.
+  CallBack* const seizeErrHand;
 };
 
 template <class T>
 inline
-ArrayPool<T>::ArrayPool(){
+ArrayPool<T>::ArrayPool(CallBack* seizeErrorHandler):
+  seizeErrHand(seizeErrorHandler)
+{
   firstFree = RNIL;
   size = 0;
   noOfFree = 0;
+  noOfFreeMin = 0;
   theArray = 0;
   alloc_ptr = 0;
 #ifdef ARRAY_GUARD
   theAllocatedBitmask = 0;
+  chunk = false;
 #endif
 }
 
@@ -269,6 +388,7 @@ ArrayPool<T>::setSize(Uint32 noOfElements,
     }
     size = noOfElements;
     noOfFree = noOfElements;
+    noOfFreeMin = noOfElements;
 
     /**
      * Set next pointers
@@ -299,6 +419,32 @@ ArrayPool<T>::setSize(Uint32 noOfElements,
   return false; // not reached
 }
 
+template <class T>
+inline
+void
+ArrayPool<T>::setChunkSize(Uint32 sz)
+{
+  Uint32 i;
+  for (i = 0; i + sz < size; i += sz)
+  {
+    theArray[i].chunkSize = sz;
+    theArray[i].lastChunk = i + sz - 1;
+    theArray[i].nextChunk = i + sz;
+  }
+
+  theArray[i].chunkSize = size - i;
+  theArray[i].lastChunk = size - 1;
+  theArray[i].nextChunk = RNIL;
+
+#ifdef ARRAY_GUARD
+  chunk = true;
+#endif
+
+#ifdef ARRAY_CHUNK_GUARD
+  checkChunks();
+#endif
+}
+
 template <class T>
 inline
 bool
@@ -324,6 +470,7 @@ ArrayPool<T>::set(T* ptr, Uint32 cnt, bool align){
     
     size = cnt;
     noOfFree = 0;
+    noOfFreeMin = 0;
     return true;
   }
   ErrorReporter::handleAssert("ArrayPool<T>::set called twice", 
@@ -639,6 +786,10 @@ template <class T>
 inline
 bool
 ArrayPool<T>::seize(Ptr<T> & ptr){
+#ifdef ARRAY_GUARD
+  assert(chunk == false);
+#endif
+
   Uint32 ff = firstFree;
   if(ff != RNIL){
     firstFree = theArray[ff].nextPool;
@@ -650,7 +801,7 @@ ArrayPool<T>::seize(Ptr<T> & ptr){
     {
       if(!BitmaskImpl::get(bitmaskSz, theAllocatedBitmask, ff)){
 	BitmaskImpl::set(bitmaskSz, theAllocatedBitmask, ff);
-	noOfFree--;
+	decNoFree();
 	return true;
       } else {
 	/**
@@ -661,11 +812,15 @@ ArrayPool<T>::seize(Ptr<T> & ptr){
       }
     }
 #endif
-    noOfFree--;
+    decNoFree();
     return true;
   }
   ptr.i = RNIL;
   ptr.p = NULL;
+  if (seizeErrHand != NULL)
+  {
+    (*seizeErrHand)(*this);
+  }
   return false;
 }
 
@@ -673,6 +828,10 @@ template <class T>
 inline
 bool
 ArrayPool<T>::seizeId(Ptr<T> & ptr, Uint32 i){
+#ifdef ARRAY_GUARD
+  assert(chunk == false);
+#endif
+
   Uint32 ff = firstFree;
   Uint32 prev = RNIL;
   while(ff != i && ff != RNIL){
@@ -693,7 +852,7 @@ ArrayPool<T>::seizeId(Ptr<T> & ptr, Uint32 i){
     {
       if(!BitmaskImpl::get(bitmaskSz, theAllocatedBitmask, ff)){
 	BitmaskImpl::set(bitmaskSz, theAllocatedBitmask, ff);
-	noOfFree--;
+	decNoFree();
 	return true;
       } else {
 	/**
@@ -704,11 +863,15 @@ ArrayPool<T>::seizeId(Ptr<T> & ptr, Uint32 i){
       }
     }
 #endif
-    noOfFree--;
+    decNoFree();
     return true;
   }
   ptr.i = RNIL;
   ptr.p = NULL;
+  if (seizeErrHand != NULL)
+  {
+    (*seizeErrHand)(*this);
+  }
   return false;
 }
 
@@ -728,6 +891,10 @@ ArrayPool<T>::findId(Uint32 i) const {
 template<class T>
 Uint32
 ArrayPool<T>::seizeN(Uint32 n){
+#ifdef ARRAY_GUARD
+  assert(chunk == false);
+#endif
+
   Uint32 curr = firstFree;
   Uint32 prev = RNIL;
   Uint32 sz = 0;
@@ -741,6 +908,10 @@ ArrayPool<T>::seizeN(Uint32 n){
     curr = theArray[curr].nextPool;
   }
   if(sz != n){
+    if (seizeErrHand != NULL)
+    {
+      (*seizeErrHand)(*this);
+    }
     return RNIL;
   }
   const Uint32 base = curr - n;
@@ -750,7 +921,7 @@ ArrayPool<T>::seizeN(Uint32 n){
     theArray[prev].nextPool = curr;
   }
   
-  noOfFree -= n;
+  decNoFree(n);
 #ifdef ARRAY_GUARD
   if (theAllocatedBitmask)
   {
@@ -774,6 +945,10 @@ template<class T>
 inline
 void 
 ArrayPool<T>::releaseN(Uint32 base, Uint32 n){
+#ifdef ARRAY_GUARD
+  assert(chunk == false);
+#endif
+
   Uint32 curr = firstFree;
   Uint32 prev = RNIL;
   while(curr < base){
@@ -812,6 +987,12 @@ inline
 void 
 ArrayPool<T>::releaseList(Uint32 n, Uint32 first, Uint32 last){
 
+#ifdef ARRAY_GUARD
+  assert(chunk == false);
+#endif
+
+  assert( n != 0 );
+
   if(first < size && last < size){
     Uint32 ff = firstFree;
     firstFree = first;
@@ -849,6 +1030,11 @@ template <class T>
 inline
 void
 ArrayPool<T>::release(Uint32 _i){
+
+#ifdef ARRAY_GUARD
+  assert(chunk == false);
+#endif
+
   const Uint32 i = _i;
   if(likely(i < size)){
     Uint32 ff = firstFree;
@@ -882,6 +1068,11 @@ template <class T>
 inline
 void
 ArrayPool<T>::release(Ptr<T> & ptr){
+
+#ifdef ARRAY_GUARD
+  assert(chunk == false);
+#endif
+
   Uint32 i = ptr.i;
   if(likely(i < size)){
     Uint32 ff = firstFree;
@@ -909,6 +1100,276 @@ ArrayPool<T>::release(Ptr<T> & ptr){
   ErrorReporter::handleAssert("ArrayPool<T>::release", __FILE__, __LINE__);
 }
 
+#if 0
+#define DUMP(a,b) do { printf("%s c.m_first_free: %u c.m_free_cnt: %u %s", a, c.m_first_free, c.m_free_cnt, b); fflush(stdout); } while(0)
+#else
+#define DUMP(a,b)
+#endif
+
+#ifdef ARRAY_CHUNK_GUARD
+template <class T>
+inline
+void
+ArrayPool<T>::checkChunks()
+{
+#ifdef ARRAY_GUARD
+  assert(chunk == true);
+#endif
+
+  Uint32 ff = firstFree;
+  Uint32 sum = 0;
+  while (ff != RNIL)
+  {
+    sum += theArray[ff].chunkSize;
+    Uint32 last = theArray[ff].lastChunk;
+    assert(theArray[last].nextPool == theArray[ff].nextChunk);
+    if (theAllocatedBitmask)
+    {
+      Uint32 tmp = ff;
+      while(tmp != last)
+      {
+        assert(!BitmaskImpl::get(bitmaskSz, theAllocatedBitmask, tmp));
+        tmp = theArray[tmp].nextPool;
+      }
+      assert(!BitmaskImpl::get(bitmaskSz, theAllocatedBitmask, tmp));
+    }
+
+    ff = theArray[ff].nextChunk;
+  }
+  assert(sum == noOfFree);
+}
+#endif
+
+template <class T>
+inline
+bool
+ArrayPool<T>::seizeChunk(Uint32 & cnt, Ptr<T> & ptr)
+{
+#ifdef ARRAY_GUARD
+  assert(chunk == true);
+#endif
+
+#ifdef ARRAY_CHUNK_GUARD
+  checkChunks();
+#endif
+
+  Uint32 save = cnt;
+  int tmp = save;
+  Uint32 ff = firstFree;
+  ptr.i = ff;
+  ptr.p = theArray + ff;
+
+  if (ff != RNIL)
+  {
+    Uint32 prev;
+    do 
+    {
+      if (0)
+        ndbout_c("seizeChunk(%u) ff: %u tmp: %d chunkSize: %u lastChunk: %u nextChunk: %u",
+                 save, ff, tmp, 
+                 theArray[ff].chunkSize, 
+                 theArray[ff].lastChunk,
+                 theArray[ff].nextChunk);
+      
+      tmp -= theArray[ff].chunkSize;
+      prev = theArray[ff].lastChunk;
+      assert(theArray[ff].nextChunk == theArray[prev].nextPool);
+      ff = theArray[ff].nextChunk;
+    } while (tmp > 0 && ff != RNIL);
+    
+    cnt = (save - tmp);
+    decNoFree(save - tmp);
+    firstFree = ff;
+    theArray[prev].nextPool = RNIL;
+    
+#ifdef ARRAY_GUARD
+    if (theAllocatedBitmask)
+    {
+      Uint32 tmpI = ptr.i;
+      for(Uint32 i = 0; i<cnt; i++){
+        if(BitmaskImpl::get(bitmaskSz, theAllocatedBitmask, tmpI)){
+          /**
+           * Seizing an already seized element
+           */
+          ErrorReporter::handleAssert("ArrayPool<T>::seizeChunk", 
+                                      __FILE__, __LINE__);
+        } else {
+          BitmaskImpl::set(bitmaskSz, theAllocatedBitmask, tmpI);
+        }
+        tmpI = theArray[tmpI].nextPool;
+      }
+    }
+#endif
+
+#ifdef ARRAY_CHUNK_GUARD
+    checkChunks();
+#endif
+
+    return true;
+  }
+
+  ptr.p = NULL;
+  if (seizeErrHand != NULL)
+  {
+    (*seizeErrHand)(*this);
+  }
+  return false;
+}
+
+template <class T>
+inline
+void
+ArrayPool<T>::releaseChunk(Uint32 cnt, Uint32 first, Uint32 last)
+{
+#ifdef ARRAY_GUARD
+  assert(chunk == true);
+#endif
+
+#ifdef ARRAY_CHUNK_GUARD
+  checkChunks();
+#endif
+
+  Uint32 ff = firstFree;
+  firstFree = first;
+  theArray[first].nextChunk = ff;
+  theArray[last].nextPool = ff;
+  noOfFree += cnt;
+
+  assert(theArray[first].chunkSize == cnt);
+  assert(theArray[first].lastChunk == last);
+  
+#ifdef ARRAY_GUARD
+  if (theAllocatedBitmask)
+  {
+    Uint32 tmp = first;
+    for(Uint32 i = 0; i<cnt; i++){
+      if(BitmaskImpl::get(bitmaskSz, theAllocatedBitmask, tmp)){
+        BitmaskImpl::clear(bitmaskSz, theAllocatedBitmask, tmp);
+      } else {
+        /**
+         * Relesing a already released element
+         */
+        ErrorReporter::handleAssert("ArrayPool<T>::releaseList", 
+                                    __FILE__, __LINE__);
+        return;
+      }
+      tmp = theArray[tmp].nextPool;
+    }
+  }
+#endif
+
+#ifdef ARRAY_CHUNK_GUARD
+  checkChunks();
+#endif
+}
+
+
+template <class T>
+inline
+bool
+ArrayPool<T>::seize(LockFun l, Cache& c, Ptr<T> & p)
+{
+  DUMP("seize", "-> ");
+
+  Uint32 ff = c.m_first_free;
+  if (ff != RNIL)
+  {
+    c.m_first_free = theArray[ff].nextPool;
+    c.m_free_cnt--;
+    p.i = ff;
+    p.p = theArray + ff;
+    DUMP("LOCAL ", "\n");
+    return true;
+  }
+
+  Uint32 tmp = c.m_alloc_cnt;
+  l.lock();
+  bool ret = seizeChunk(tmp, p);
+  l.unlock();
+
+  if (ret)
+  {
+    c.m_first_free = theArray[p.i].nextPool;
+    c.m_free_cnt = tmp - 1;
+    DUMP("LOCKED", "\n");
+    return true;
+  }
+  if (seizeErrHand != NULL)
+  {
+    (*seizeErrHand)(*this);
+  }
+  return false;
+}
+
+template <class T>
+inline
+void
+ArrayPool<T>::release(LockFun l, Cache& c, Uint32 i)
+{
+  Ptr<T> tmp;
+  getPtr(tmp, i);
+  release(l, c, tmp);
+}
+
+template <class T>
+inline
+void
+ArrayPool<T>::release(LockFun l, Cache& c, Ptr<T> & p)
+{
+  p.p->nextPool = c.m_first_free;
+  c.m_first_free = p.i;
+  c.m_free_cnt ++;
+
+  if (c.m_free_cnt > 2 * c.m_max_free_cnt)
+  {
+    releaseChunk(l, c, c.m_alloc_cnt);
+  }
+}
+
+template <class T>
+inline
+void
+ArrayPool<T>::releaseList(LockFun l, Cache& c,
+                          Uint32 n, Uint32 first, Uint32 last)
+{
+  theArray[last].nextPool = c.m_first_free;
+  c.m_first_free = first;
+  c.m_free_cnt += n;
+
+  if (c.m_free_cnt > 2 * c.m_max_free_cnt)
+  {
+    releaseChunk(l, c, c.m_alloc_cnt);
+  }
+}
+
+template <class T>
+inline
+void
+ArrayPool<T>::releaseChunk(LockFun l, Cache& c, Uint32 n)
+{
+  DUMP("releaseListImpl", "-> ");
+  Uint32 ff = c.m_first_free;
+  Uint32 prev = ff;
+  Uint32 curr = ff;
+  Uint32 i;
+  for (i = 0; i < n && curr != RNIL; i++)
+  {
+    prev = curr;
+    curr = theArray[curr].nextPool;
+  }
+  c.m_first_free = curr;
+  c.m_free_cnt -= i;
+
+  DUMP("", "\n");
+
+  theArray[ff].chunkSize = i;
+  theArray[ff].lastChunk = prev;
+
+  l.lock();
+  releaseChunk(i, ff, prev);
+  l.unlock();
+}
+
 template <class T>
 class UnsafeArrayPool : public ArrayPool<T> {
 public:
@@ -1004,5 +1465,112 @@ UnsafeArrayPool<T>::getPtrForce(ConstPtr<T> & ptr, Uint32 i) const{
   }
 }
 
+// SafeArrayPool
+
+template <class T>
+class SafeArrayPool : public ArrayPool<T> {
+public:
+  SafeArrayPool();
+  ~SafeArrayPool();
+  int lock();
+  int unlock();
+  bool seize(Ptr<T>&);
+  void release(Uint32 i);
+  void release(Ptr<T>&);
+
+  void setMutex(NdbMutex* mutex = 0);
+
+private:
+  NdbMutex* m_mutex;
+  bool m_mutex_owner;
+
+  bool seizeId(Ptr<T>&);
+  bool findId(Uint32) const;
+};
+
+template <class T>
+inline
+SafeArrayPool<T>::SafeArrayPool()
+{
+  m_mutex = 0;
+  m_mutex_owner = false;
+}
+
+template <class T>
+inline
+void
+SafeArrayPool<T>::setMutex(NdbMutex* mutex)
+{
+  if (mutex != 0) {
+    m_mutex = mutex;
+    m_mutex_owner = false;
+  } else {
+    m_mutex = NdbMutex_Create();
+    assert(m_mutex != 0);
+    m_mutex_owner = true;
+  }
+}
+
+template <class T>
+inline
+SafeArrayPool<T>::~SafeArrayPool()
+{
+  if (m_mutex_owner)
+    NdbMutex_Destroy(m_mutex);
+}
+
+template <class T>
+inline
+int
+SafeArrayPool<T>::lock()
+{
+  return NdbMutex_Lock(m_mutex);
+}
+
+template <class T>
+inline
+int
+SafeArrayPool<T>::unlock()
+{
+  return NdbMutex_Unlock(m_mutex);
+}
+
+template <class T>
+inline
+bool
+SafeArrayPool<T>::seize(Ptr<T>& ptr)
+{
+  bool ok = false;
+  if (lock() == 0) {
+    ok = ArrayPool<T>::seize(ptr);
+    unlock();
+  }
+  return ok;
+}
+
+template <class T>
+inline
+void
+SafeArrayPool<T>::release(Uint32 i)
+{
+  int ret = lock();
+  assert(ret == 0);
+  ArrayPool<T>::release(i);
+  unlock();
+}
+
+template <class T>
+inline
+void
+SafeArrayPool<T>::release(Ptr<T>& ptr)
+{
+  int ret = lock();
+  assert(ret == 0);
+  ArrayPool<T>::release(ptr);
+  unlock();
+}
+
+
+#undef JAM_FILE_ID
 
 #endif
