@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2003, 2010, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2015, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -17,11 +24,11 @@
 
 #include <HugoOperations.hpp>
 
-#undef ERR
-#define ERR(error) \
+#undef NDB_ERR
+#define NDB_ERR(error) \
 { \
   const NdbError &_error= (error); \
-  if (!m_quiet) ERR_OUT(g_err, _error); \
+  if (!m_quiet) NDB_ERR_OUT(g_err, _error); \
 }
 
 int HugoOperations::startTransaction(Ndb* pNdb,
@@ -35,7 +42,26 @@ int HugoOperations::startTransaction(Ndb* pNdb,
   pTrans = pNdb->startTransaction(table, keyData, keyLen);
   if (pTrans == NULL) {
     const NdbError err = pNdb->getNdbError();
-    ERR(err);
+    NDB_ERR(err);
+    setNdbError(err);
+    return NDBT_FAILED;
+  }
+  return NDBT_OK;
+}
+
+int HugoOperations::startTransaction(Ndb* pNdb,
+                                     Uint32 node_id,
+                                     Uint32 instance_id)
+{
+  if (pTrans != NULL)
+  {
+    ndbout << "HugoOperations::startTransaction, pTrans != NULL" << endl;
+    return NDBT_FAILED;
+  }
+  pTrans = pNdb->startTransaction(node_id, instance_id);
+  if (pTrans == NULL) {
+    const NdbError err = pNdb->getNdbError();
+    NDB_ERR(err);
     setNdbError(err);
     return NDBT_FAILED;
   }
@@ -45,11 +71,12 @@ int HugoOperations::startTransaction(Ndb* pNdb,
 int HugoOperations::setTransaction(NdbTransaction* new_trans, bool not_null_ok){
   
   if (pTrans != NULL && !not_null_ok){
-    ndbout << "HugoOperations::startTransaction, pTrans != NULL" << endl;
+    ndbout << "HugoOperations::setTransaction, pTrans != NULL" << endl;
     return NDBT_FAILED;
   }
   pTrans = new_trans;
   if (pTrans == NULL) {
+    ndbout << "HugoOperations::setTransaction, pTrans == NULL" << endl;
     return NDBT_FAILED;
   }
   return NDBT_OK;
@@ -69,6 +96,7 @@ int HugoOperations::closeTransaction(Ndb* pNdb){
   m_result_sets.clear();
   m_executed_result_sets.clear();
 
+  pTrans = NULL;
   return NDBT_OK;
 }
 
@@ -96,7 +124,7 @@ int HugoOperations::pkReadRecord(Ndb* pNdb,
       pOp = getOperation(pTrans, NdbOperation::ReadRequest);
     }
     if (pOp == NULL) {
-      ERR(pTrans->getNdbError());
+      NDB_ERR(pTrans->getNdbError());
       setNdbError(pTrans->getNdbError());
       return NDBT_FAILED;
     }
@@ -130,14 +158,17 @@ rand_lock_mode:
     }
     
     if( check == -1 ) {
-      ERR(pTrans->getNdbError());
+      NDB_ERR(pTrans->getNdbError());
       setNdbError(pTrans->getNdbError());
       return NDBT_FAILED;
     }
     
     // Define primary keys
     if (equalForRow(pOp, r+recordNo) != 0)
+    {
+      g_err << __LINE__ << " equal for row failed" << endl;
       return NDBT_FAILED;
+    }
 
     Uint32 partId;
     /* Do we need to set the partitionId for this operation? */
@@ -155,8 +186,8 @@ rand_lock_mode:
       // Define attributes to read  
       for(a = 0; a<tab.getNoOfColumns(); a++){
 	if((rows[r]->attributeStore(a) = 
-	    pOp->getValue(tab.getColumn(a)->getName())) == 0) {
-	  ERR(pTrans->getNdbError());
+	    pOp->getValue(tab.getColumn(a))) == 0) {
+	  NDB_ERR(pTrans->getNdbError());
           setNdbError(pTrans->getNdbError());
 	  return NDBT_FAILED;
 	}
@@ -190,7 +221,7 @@ int HugoOperations::pkReadRandRecord(Ndb* pNdb,
       pOp = getOperation(pTrans, NdbOperation::ReadRequest);
     }
     if (pOp == NULL) {
-      ERR(pTrans->getNdbError());
+      NDB_ERR(pTrans->getNdbError());
       setNdbError(pTrans->getNdbError());
       return NDBT_FAILED;
     }
@@ -220,7 +251,7 @@ rand_lock_mode:
     }
     
     if( check == -1 ) {
-      ERR(pTrans->getNdbError());
+      NDB_ERR(pTrans->getNdbError());
       setNdbError(pTrans->getNdbError());
       return NDBT_FAILED;
     }
@@ -229,7 +260,10 @@ rand_lock_mode:
 
     // Define primary keys
     if (equalForRow(pOp, rowid) != 0)
+    {
+      g_err << __LINE__ << " equal for row failed" << endl;
       return NDBT_FAILED;
+    }
 
     Uint32 partId;
     /* Do we need to set the partitionId for this operation? */
@@ -247,8 +281,8 @@ rand_lock_mode:
       // Define attributes to read  
       for(a = 0; a<tab.getNoOfColumns(); a++){
 	if((rows[r]->attributeStore(a) = 
-	    pOp->getValue(tab.getColumn(a)->getName())) == 0) {
-	  ERR(pTrans->getNdbError());
+	    pOp->getValue(tab.getColumn(a))) == 0) {
+	  NDB_ERR(pTrans->getNdbError());
           setNdbError(pTrans->getNdbError());
 	  return NDBT_FAILED;
 	}
@@ -262,6 +296,115 @@ rand_lock_mode:
   return NDBT_OK;
 }
 
+int HugoOperations::pkReadRecordLockHandle(Ndb* pNdb,
+                                           Vector<const NdbLockHandle*>& lockHandles,
+                                           int recordNo,
+                                           int numRecords,
+                                           NdbOperation::LockMode lm,
+                                           NdbOperation::LockMode *lmused){
+  if (idx)
+  {
+    g_err << "ERROR : Cannot call pkReadRecordLockHandle on an index"
+          << endl;
+    return NDBT_FAILED;
+  }
+
+  /* If something other than LM_Read or LM_Exclusive is 
+   * passed in then we'll choose, and PkReadRecord
+   * will update lm_used
+   */
+  while (lm != NdbOperation::LM_Read &&
+         lm != NdbOperation::LM_Exclusive)
+  {
+    lm = (NdbOperation::LockMode)((rand() >> 16) & 1);
+  }
+
+  const NdbOperation* prevOp = pTrans->getLastDefinedOperation();
+
+  int readRc = pkReadRecord(pNdb,
+                            recordNo,
+                            numRecords,
+                            lm,
+                            lmused);
+
+  if (readRc == NDBT_OK)
+  {
+    /* Now traverse operations added, requesting
+     * LockHandles
+     */
+    const NdbOperation* definedOp = (prevOp)? prevOp->next() : 
+      pTrans->getFirstDefinedOperation();
+    
+    while (definedOp)
+    {
+      /* Look away now */
+      const NdbLockHandle* lh = 
+        (const_cast<NdbOperation*>(definedOp))->getLockHandle();
+      
+      if (lh == NULL)
+      {
+        NDB_ERR(definedOp->getNdbError());
+        setNdbError(definedOp->getNdbError());
+        return NDBT_FAILED;
+      }
+
+      lockHandles.push_back(lh);
+      definedOp = definedOp->next();
+    }
+  }
+
+  return readRc;
+}
+
+int HugoOperations::pkUnlockRecord(Ndb* pNdb,
+                                   Vector<const NdbLockHandle*>& lockHandles,
+                                   int offset,
+                                   int numRecords,
+                                   NdbOperation::AbortOption ao)
+{
+  if (numRecords == ~(0))
+  {
+    numRecords = lockHandles.size() - offset;
+  }
+
+  if (lockHandles.size() < (unsigned)(offset + numRecords))
+  {
+    g_err << "ERROR : LockHandles size is "
+          << lockHandles.size()
+          << " offset (" 
+          << offset
+          << ") and/or numRecords ("
+          << numRecords
+          << ") too large." << endl;
+    return NDBT_FAILED;
+  }
+  
+  for (int i = 0; i < numRecords; i++)
+  {
+    const NdbLockHandle* lh = lockHandles[offset + i];
+    if (lh != NULL)
+    {
+      const NdbOperation* unlockOp = pTrans->unlock(lh, ao);
+      
+      if (unlockOp == NULL)
+      {
+        NDB_ERR(pTrans->getNdbError());
+        setNdbError(pTrans->getNdbError());
+        return NDBT_FAILED;
+      }
+    }
+    else
+    {
+      g_err << "ERROR : LockHandle number "
+            << i+offset << " is NULL. "
+            << " offset is " << offset << endl;
+      return NDBT_FAILED;
+    }
+  }
+
+  return NDBT_OK;
+}
+
 int HugoOperations::pkUpdateRecord(Ndb* pNdb,
 				   int recordNo,
 				   int numRecords,
@@ -271,26 +414,29 @@ int HugoOperations::pkUpdateRecord(Ndb* pNdb,
   for(int r=0; r < numRecords; r++){
     NdbOperation* pOp = getOperation(pTrans, NdbOperation::UpdateRequest);
     if (pOp == NULL) {
-      ERR(pTrans->getNdbError());
+      NDB_ERR(pTrans->getNdbError());
       setNdbError(pTrans->getNdbError());
       return NDBT_FAILED;
     }
     
     check = pOp->updateTuple();
     if( check == -1 ) {
-      ERR(pTrans->getNdbError());
+      NDB_ERR(pTrans->getNdbError());
       setNdbError(pTrans->getNdbError());
       return NDBT_FAILED;
     }
     
     if(setValues(pOp, r+recordNo, updatesValue) != NDBT_OK)
     {
+      g_err << __LINE__ << " setValues failed" << endl;
       return NDBT_FAILED;
     }
 
     Uint32 partId;
     if(getPartIdForRow(pOp, r+recordNo, partId))
       pOp->setPartitionId(partId);
+
+    pOp->setAnyValue(getAnyValueForRowUpd(r+recordNo, updatesValue));
     
   }
   return NDBT_OK;
@@ -300,20 +446,36 @@ int
 HugoOperations::setValues(NdbOperation* pOp, int rowId, int updateId)
 {
   // Define primary keys
-  int a;
   if (equalForRow(pOp, rowId) != 0)
+  {
+    g_err << __LINE__ << " equal for row failed" << endl;
     return NDBT_FAILED;
-  
-  for(a = 0; a<tab.getNoOfColumns(); a++){
-    if (tab.getColumn(a)->getPrimaryKey() == false){
-      if(setValueForAttr(pOp, a, rowId, updateId ) != 0){ 
-	ERR(pTrans->getNdbError());
+  }
+
+  if (setNonPkValues(pOp, rowId, updateId) != 0)
+  {
+    g_err << __LINE__ << " setNonPkValues failed" << endl;
+    return NDBT_FAILED;
+  }
+
+  return NDBT_OK;
+}
+
+int
+HugoOperations::setNonPkValues(NdbOperation* pOp, int rowId, int updateId)
+{
+  for(int a = 0; a<tab.getNoOfColumns(); a++)
+  {
+    if (tab.getColumn(a)->getPrimaryKey() == false)
+    {
+      if(setValueForAttr(pOp, a, rowId, updateId ) != 0)
+      {
+	NDB_ERR(pTrans->getNdbError());
         setNdbError(pTrans->getNdbError());
 	return NDBT_FAILED;
       }
     }
   }
-  
   return NDBT_OK;
 }
 
@@ -326,14 +488,14 @@ int HugoOperations::pkInsertRecord(Ndb* pNdb,
   for(int r=0; r < numRecords; r++){
     NdbOperation* pOp = getOperation(pTrans, NdbOperation::InsertRequest);
     if (pOp == NULL) {
-      ERR(pTrans->getNdbError());
+      NDB_ERR(pTrans->getNdbError());
       setNdbError(pTrans->getNdbError());
       return NDBT_FAILED;
     }
     
     check = pOp->insertTuple();
     if( check == -1 ) {
-      ERR(pTrans->getNdbError());
+      NDB_ERR(pTrans->getNdbError());
       setNdbError(pTrans->getNdbError());
       return NDBT_FAILED;
     }
@@ -341,6 +503,7 @@ int HugoOperations::pkInsertRecord(Ndb* pNdb,
     if(setValues(pOp, r+recordNo, updatesValue) != NDBT_OK)
     {
       m_error.code = pTrans->getNdbError().code;
+      g_err << __LINE__ << " setValues failed" << endl;
       return NDBT_FAILED;
     }
 
@@ -356,26 +519,28 @@ int HugoOperations::pkWriteRecord(Ndb* pNdb,
 				  int recordNo,
 				  int numRecords,
 				  int updatesValue){
-  
   int a, check;
   for(int r=0; r < numRecords; r++){
     NdbOperation* pOp = pTrans->getNdbOperation(tab.getName());	
     if (pOp == NULL) {
-      ERR(pTrans->getNdbError());
+      NDB_ERR(pTrans->getNdbError());
       setNdbError(pTrans->getNdbError());
       return NDBT_FAILED;
     }
     
     check = pOp->writeTuple();
     if( check == -1 ) {
-      ERR(pTrans->getNdbError());
+      NDB_ERR(pTrans->getNdbError());
       setNdbError(pTrans->getNdbError());
       return NDBT_FAILED;
     }
     
     // Define primary keys
     if (equalForRow(pOp, r+recordNo) != 0)
+    {
+      g_err << __LINE__ << " equal for row failed" << endl;
       return NDBT_FAILED;
+    }
     
     Uint32 partId;
     if(getPartIdForRow(pOp, r+recordNo, partId))
@@ -386,7 +551,7 @@ int HugoOperations::pkWriteRecord(Ndb* pNdb,
     for(a = 0; a<tab.getNoOfColumns(); a++){
       if (tab.getColumn(a)->getPrimaryKey() == false){
 	if(setValueForAttr(pOp, a, recordNo+r, updatesValue ) != 0){ 
-	  ERR(pTrans->getNdbError());
+	  NDB_ERR(pTrans->getNdbError());
           setNdbError(pTrans->getNdbError());
 	  return NDBT_FAILED;
 	}
@@ -404,21 +569,24 @@ int HugoOperations::pkWritePartialRecord(Ndb* pNdb,
   for(int r=0; r < numRecords; r++){
     NdbOperation* pOp = pTrans->getNdbOperation(tab.getName());	
     if (pOp == NULL) {
-      ERR(pTrans->getNdbError());
+      NDB_ERR(pTrans->getNdbError());
       setNdbError(pTrans->getNdbError());
       return NDBT_FAILED;
     }
     
     check = pOp->writeTuple();
     if( check == -1 ) {
-      ERR(pTrans->getNdbError());
+      NDB_ERR(pTrans->getNdbError());
       setNdbError(pTrans->getNdbError());
       return NDBT_FAILED;
     }
     
     // Define primary keys
     if (equalForRow(pOp, r+recordNo) != 0)
+    {
+      g_err << __LINE__ << " equal for row failed" << endl;
       return NDBT_FAILED;
+    }
 
     Uint32 partId;
     if(getPartIdForRow(pOp, r+recordNo, partId))
@@ -436,21 +604,24 @@ int HugoOperations::pkDeleteRecord(Ndb* pNdb,
   for(int r=0; r < numRecords; r++){
     NdbOperation* pOp = getOperation(pTrans, NdbOperation::DeleteRequest);
     if (pOp == NULL) {
-      ERR(pTrans->getNdbError());
+      NDB_ERR(pTrans->getNdbError());
       setNdbError(pTrans->getNdbError());
       return NDBT_FAILED;
     }
     
     check = pOp->deleteTuple();
     if( check == -1 ) {
-      ERR(pTrans->getNdbError());
+      NDB_ERR(pTrans->getNdbError());
       setNdbError(pTrans->getNdbError());
       return NDBT_FAILED;
     }
     
     // Define primary keys
     if (equalForRow(pOp, r+recordNo) != 0)
+    {
+      g_err << __LINE__ << " equal for row failed" << endl;
       return NDBT_FAILED;
+    }
 
     Uint32 partId;
     if(getPartIdForRow(pOp, r+recordNo, partId))
@@ -459,6 +630,49 @@ int HugoOperations::pkDeleteRecord(Ndb* pNdb,
   return NDBT_OK;
 }
 
+int HugoOperations::pkRefreshRecord(Ndb* pNdb,
+                                    int recordNo,
+                                    int numRecords,
+                                    int anyValueInfo){
+
+  char buffer[NDB_MAX_TUPLE_SIZE];
+  const NdbDictionary::Table * pTab =
+    pNdb->getDictionary()->getTable(tab.getName());
+
+  if (pTab == 0)
+  {
+    g_err << __LINE__ << " pTab == 0" << endl;
+    return NDBT_FAILED;
+  }
+
+  const NdbRecord * record = pTab->getDefaultRecord();
+  NdbOperation::OperationOptions opts;
+  opts.optionsPresent = NdbOperation::OperationOptions::OO_ANYVALUE;
+  for(int r=0; r < numRecords; r++)
+  {
+    bzero(buffer, sizeof(buffer));
+    if (calc.equalForRow((Uint8*)buffer, record, r + recordNo))
+    {
+      g_err << __LINE__ << " equal for row failed" << endl;
+      return NDBT_FAILED;
+    }
+
+    opts.anyValue = anyValueInfo?
+      (anyValueInfo << 16) | (r+recordNo) :
+      0;
+
+    const NdbOperation* pOp = pTrans->refreshTuple(record, buffer,
+                                                   &opts, sizeof(opts));
+    if (pOp == NULL)
+    {
+      NDB_ERR(pTrans->getNdbError());
+      setNdbError(pTrans->getNdbError());
+      return NDBT_FAILED;
+    }
+  }
+  return NDBT_OK;
+}
+
 int HugoOperations::execute_Commit(Ndb* pNdb,
 				   AbortOption eao){
 
@@ -467,16 +681,19 @@ int HugoOperations::execute_Commit(Ndb* pNdb,
 
   const NdbError err = pTrans->getNdbError();
   if( check == -1 || err.code) {
-    ERR(err);
+    NDB_ERR(err);
     setNdbError(err);
     NdbOperation* pOp = pTrans->getNdbErrorOperation();
     if (pOp != NULL){
       const NdbError err2 = pOp->getNdbError();
-      ERR(err2);
+      NDB_ERR(err2);
       setNdbError(err2);
     }
     if (err.code == 0)
+    {
+      g_err << __LINE__ << " execute_Commit failed with errcode = 0" << endl;
       return NDBT_FAILED;
+    }
     return err.code;
   }
 
@@ -491,7 +708,7 @@ int HugoOperations::execute_Commit(Ndb* pNdb,
       return 626;
     case -1:
       const NdbError err = pTrans->getNdbError();
-      ERR(err);
+      NDB_ERR(err);
       setNdbError(err);
       return (err.code > 0 ? err.code : NDBT_FAILED);
     }
@@ -519,7 +736,7 @@ int HugoOperations::execute_NoCommit(Ndb* pNdb, AbortOption eao){
 
   const NdbError err = pTrans->getNdbError();
   if( check == -1 || err.code) {
-    ERR(err);
+    NDB_ERR(err);
     setNdbError(err);
     const NdbOperation* pOp = pTrans->getNdbErrorOperation();
     while (pOp != NULL)
@@ -527,13 +744,16 @@ int HugoOperations::execute_NoCommit(Ndb* pNdb, AbortOption eao){
       const NdbError err2 = pOp->getNdbError();
       if (err2.code)
       {
-	ERR(err2);
+	NDB_ERR(err2);
         setNdbError(err2);
       }
       pOp = pTrans->getNextCompletedOperation(pOp);
     }
     if (err.code == 0)
+    {
+      g_err << __LINE__ << " equal for row failed" << endl;
       return NDBT_FAILED;
+    }
     return err.code;
   }
 
@@ -548,7 +768,7 @@ int HugoOperations::execute_NoCommit(Ndb* pNdb, AbortOption eao){
       return 626;
     case -1:
       const NdbError err = pTrans->getNdbError();
-      ERR(err);
+      NDB_ERR(err);
       setNdbError(err);
       return (err.code > 0 ? err.code : NDBT_FAILED);
     }
@@ -574,7 +794,7 @@ int HugoOperations::execute_Rollback(Ndb* pNdb){
   check = pTrans->execute(Rollback);   
   if( check == -1 ) {
     const NdbError err = pTrans->getNdbError();
-    ERR(err);
+    NDB_ERR(err);
     setNdbError(err);
     return NDBT_FAILED;
   }
@@ -654,7 +874,8 @@ HugoOperations::HugoOperations(const NdbDictionary::Table& _tab,
   UtilTransactions(_tab, idx),
   pIndexScanOp(NULL),
   calc(_tab),
-  m_quiet(false)
+  m_quiet(false),
+  avCallback(NULL)
 {
 }
 
@@ -676,7 +897,7 @@ HugoOperations::equalForRow(NdbOperation* pOp, int row)
     {
       if(equalForAttr(pOp, a, row) != 0)
       {
-        ERR(pOp->getNdbError());
+        NDB_ERR(pOp->getNdbError());
         setNdbError(pOp->getNdbError());
         return NDBT_FAILED;
       }
@@ -720,12 +941,12 @@ int HugoOperations::equalForAttr(NdbOperation* pOp,
 				   int rowId){
   const NdbDictionary::Column* attr = tab.getColumn(attrId);  
   if (attr->getPrimaryKey() == false){
-    g_info << "Can't call equalForAttr on non PK attribute" << endl;
+    g_err << "Can't call equalForAttr on non PK attribute" << endl;
     return NDBT_FAILED;
   }
   
   int len = attr->getSizeInBytes();
-  char buf[8000];
+  char buf[NDB_MAX_TUPLE_SIZE];
   memset(buf, 0, sizeof(buf));
   Uint32 real_len;
   const char * value = calc.calcValue(rowId, attrId, 0, buf, len, &real_len);
@@ -741,7 +962,7 @@ int HugoOperations::setValueForAttr(NdbOperation* pOp,
   if (! (attr->getType() == NdbDictionary::Column::Blob))
   {
     int len = attr->getSizeInBytes();
-    char buf[8000];
+    char buf[NDB_MAX_TUPLE_SIZE];
     memset(buf, 0, sizeof(buf));
     Uint32 real_len;
     const char * value = calc.calcValue(rowId, attrId,
@@ -819,7 +1040,10 @@ void HugoOperations::deallocRows(){
 int HugoOperations::saveCopyOfRecord(int numRecords ){
 
   if (numRecords > (int)rows.size())
+  {
+    g_err << __LINE__ << " number of rows wrong" << endl;
     return NDBT_FAILED;
+  }
 
   for (int i = 0; i < numRecords; i++){
     savedRecords.push_back(rows[i]->c_str());    
@@ -840,9 +1064,15 @@ int HugoOperations::getRecordGci(int recordNum){
 
 int HugoOperations::compareRecordToCopy(int numRecords ){
   if (numRecords > (int)rows.size())
+  {
+    g_err << __LINE__ << " number of rows wrong" << endl;
     return NDBT_FAILED;
+  }
   if ((unsigned)numRecords > savedRecords.size())
+  {
+    g_err << __LINE__ << " number of rows wrong" << endl;
     return NDBT_FAILED;
+  }
 
   int result = NDBT_OK;
   for (int i = 0; i < numRecords; i++){
@@ -852,6 +1082,7 @@ int HugoOperations::compareRecordToCopy(int numRecords ){
     if (savedRecords[i] == str){
       ;
     } else {
+      g_err << __LINE__ << " row " << i << " wrong" << endl;
       result = NDBT_FAILED;
     }    
   }
@@ -875,7 +1106,7 @@ int HugoOperations::indexReadRecords(Ndb*, const char * idxName, int recordNo,
   for(int r=0; r < numRecords; r++){
     NdbOperation* pOp = pTrans->getNdbIndexOperation(idxName, tab.getName());
     if (pOp == NULL) {
-      ERR(pTrans->getNdbError());
+      NDB_ERR(pTrans->getNdbError());
       setNdbError(pTrans->getNdbError());
       return NDBT_FAILED;
     }
@@ -885,20 +1116,23 @@ int HugoOperations::indexReadRecords(Ndb*, const char * idxName, int recordNo,
     else
       check = pOp->readTuple();
     if( check == -1 ) {
-      ERR(pTrans->getNdbError());
+      NDB_ERR(pTrans->getNdbError());
       setNdbError(pTrans->getNdbError());
       return NDBT_FAILED;
     }
     
     // Define primary keys
     if (equalForRow(pOp, r+recordNo) != 0)
+    {
+      g_err << __LINE__ << " equal for row failed" << endl;
       return NDBT_FAILED;
+    }
     
     // Define attributes to read  
     for(a = 0; a<tab.getNoOfColumns(); a++){
       if((rows[r]->attributeStore(a) = 
-	  pOp->getValue(tab.getColumn(a)->getName())) == 0) {
-	ERR(pTrans->getNdbError());
+	  pOp->getValue(tab.getColumn(a))) == 0) {
+	NDB_ERR(pTrans->getNdbError());
         setNdbError(pTrans->getNdbError());
 	return NDBT_FAILED;
       }
@@ -919,27 +1153,30 @@ HugoOperations::indexUpdateRecord(Ndb*,
   for(int r=0; r < numRecords; r++){
     NdbOperation* pOp = pTrans->getNdbIndexOperation(idxName, tab.getName());
     if (pOp == NULL) {
-      ERR(pTrans->getNdbError());
+      NDB_ERR(pTrans->getNdbError());
       setNdbError(pTrans->getNdbError());
       return NDBT_FAILED;
     }
     
     check = pOp->updateTuple();
     if( check == -1 ) {
-      ERR(pTrans->getNdbError());
+      NDB_ERR(pTrans->getNdbError());
       setNdbError(pTrans->getNdbError());
       return NDBT_FAILED;
     }
     
     // Define primary keys
     if (equalForRow(pOp, r+recordNo) != 0)
+    {
+      g_err << __LINE__ << " equal for row failed" << endl;
       return NDBT_FAILED;
+    }
     
     // Define attributes to update
     for(a = 0; a<tab.getNoOfColumns(); a++){
       if (tab.getColumn(a)->getPrimaryKey() == false){
 	if(setValueForAttr(pOp, a, recordNo+r, updatesValue ) != 0){ 
-	  ERR(pTrans->getNdbError());
+	  NDB_ERR(pTrans->getNdbError());
           setNdbError(pTrans->getNdbError());
 	  return NDBT_FAILED;
 	}
@@ -965,8 +1202,8 @@ HugoOperations::scanReadRecords(Ndb* pNdb, NdbScanOperation::LockMode lm,
   
   for(int a = 0; a<tab.getNoOfColumns(); a++){
     if((rows[0]->attributeStore(a) = 
-	pOp->getValue(tab.getColumn(a)->getName())) == 0) {
-      ERR(pTrans->getNdbError());
+	pOp->getValue(tab.getColumn(a))) == 0) {
+      NDB_ERR(pTrans->getNdbError());
       setNdbError(pTrans->getNdbError());
       return NDBT_FAILED;
     }
@@ -978,6 +1215,58 @@ HugoOperations::scanReadRecords(Ndb* pNdb, NdbScanOperation::LockMode lm,
   return 0;
 }
 
+int
+HugoOperations::releaseLockHandles(Ndb* pNdb,
+                                   Vector<const NdbLockHandle*>& lockHandles,
+                                   int offset,
+                                   int numRecords)
+{
+  int totalSize = lockHandles.size();
+  if (numRecords == ~(0))
+  {
+    numRecords = totalSize - offset;
+  }
+
+  if (totalSize < (offset + numRecords))
+  {
+    g_err << "ERROR : LockHandles size is "
+          << lockHandles.size()
+          << " offset (" 
+          << offset
+          << ") and/or numRecords ("
+          << numRecords
+          << ") too large." << endl;
+    return NDBT_FAILED;
+  }
+
+  for (int i = 0; i < numRecords; i++)
+  {
+    const NdbLockHandle* lh = lockHandles[offset + i];
+    if (lh != NULL)
+    {
+      if (pTrans->releaseLockHandle(lh) != 0)
+      {
+        NDB_ERR(pTrans->getNdbError());
+        setNdbError(pTrans->getNdbError());
+        return NDBT_FAILED;
+      }
+      const NdbLockHandle* nullPtr = NULL;
+      (void)nullPtr; // ??
+      //lockHandles.set(nullPtr, offset + i, nullPtr);
+    }
+    else
+    {
+      g_err << "ERROR : LockHandle number "
+            << i+offset << " is NULL. "
+            << " offset is " << offset << endl;
+      return NDBT_FAILED;
+    }
+  }
+
+  return NDBT_OK;
+
+}
+
 static void
 update(const NdbError & _err)
 {
@@ -1000,4 +1289,21 @@ HugoOperations::setNdbError(const NdbError& error)
   m_error.code = error.code ? error.code : 1;
 }
 
+void
+HugoOperations::setAnyValueCallback(AnyValueCallback avc)
+{
+  avCallback = avc;
+}
+
+Uint32
+HugoOperations::getAnyValueForRowUpd(int row, int update)
+{
+  if (avCallback == NULL)
+    return 0;
+
+  return (avCallback)(pTrans->getNdb(), pTrans,
+                      row, update);
+}
+
 template class Vector<HugoOperations::RsPair>;
+template class Vector<const NdbLockHandle*>;
